@@ -1,5 +1,5 @@
 #ifndef lint
-static const char rcsid[] = "$Id: host.c,v 8.42 2000/12/23 08:14:32 vixie Exp $";
+static const char rcsid[] = "$Id: host.c,v 8.53 2002/06/18 02:34:02 marka Exp $";
 #endif /* not lint */
 
 /*
@@ -100,7 +100,6 @@ static const char copyright[] =
 
 #include <ctype.h>
 #include <netdb.h>
-#include <resolv.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -113,10 +112,15 @@ static const char copyright[] =
 
 #include "port_after.h"
 
+#include <resolv.h>
+
 /* Global. */
 
+#ifndef PATH_SEP
+#define PATH_SEP '/'
+#endif
 #define SIG_RDATA_BY_NAME	18
-#define NS_HEADERDATA_SIZE 10
+#define NS_HEADERDATA_SIZE 10	/* type + class + ttl + rdlen */
 
 #define NUMNS		8
 #define NUMNSADDR	16
@@ -182,15 +186,14 @@ static char		getdomain[NS_MAXDNAME];
 
 static int		parsetype(const char *s);
 static int		parseclass(const char *s);
-static void		printanswer(const struct hostent *hp);
 static void		hperror(int errnum);
-static int		addrinfo(struct in_addr addr);
+static int		addrinfo(struct sockaddr_storage *addr);
 static int		gethostinfo(char *name);
 static int		getdomaininfo(const char *name, const char *domain);
 static int		getinfo(const char *name, const char *domain,
 				int type);
 static int		printinfo(const querybuf *answer, const u_char *eom,
-				  int filter, int isls);
+				  int filter, int isls, int isinaddr);
 static const u_char *	pr_rr(const u_char *cp, const u_char *msg, FILE *file,
 			      int filter);
 static const char *	pr_type(int type);
@@ -222,16 +225,20 @@ Usage: %s [-adlrwv] [-t querytype] [-c class] host [server]\n\
 
 int
 main(int argc, char **argv) {
-	struct in_addr addr;
+	struct sockaddr_storage addr;
 	struct hostent *hp;
 	char *s;
 	int waitmode = 0;
 	int ncnames, ch;
 	int nkeychains;
+	struct addrinfo *answer = NULL;
+	struct addrinfo *cur = NULL;
+	struct addrinfo hint;
+	int ip = 0;
 
 	dst_init();
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	if ((progname = strrchr(argv[0], PATH_SEP)) == NULL)
 		progname = argv[0];
 	else
 		progname++;
@@ -289,33 +296,93 @@ main(int argc, char **argv) {
 	if (argc > 1)
 		usage("extra undefined arguments");
 	if (argc == 1) {
+		union res_sockaddr_union u[MAXNS];
+		int nscount;
+
 		s = *argv++;
 		argc--;
 		server_specified++;
+		memset(&hint, 0, sizeof(hint));
+		hint.ai_flags = AI_CANONNAME;
+		hint.ai_family = PF_UNSPEC;
+		hint.ai_socktype = SOCK_DGRAM;
 		
-		if (!inet_aton(s, &addr)) {
-			hp = gethostbyname(s);
-			if (hp == NULL) {
-				fprintf(stderr,
-					"Error in looking up server name:\n");
-				hperror(res.res_h_errno);
-				exit(1);
+		if (!getaddrinfo(s, NULL, &hint, &answer)) {
+			nscount = 0;
+			if (answer->ai_canonname != NULL) {
+				printf("Using domain server:\n");
+				printf("Name: %s\n", answer->ai_canonname);
+				printf("Addresses:");
+			} else
+				printf("Using domain server");
+			
+			for (cur = answer; cur != NULL; cur = cur->ai_next) {
+				char buf[80];
+				struct sockaddr_in6 *sin6;
+				struct sockaddr_in *sin;
+				
+				switch (cur->ai_addr->sa_family) {
+				case AF_INET6:
+					sin6 = 
+					   (struct sockaddr_in6 *)cur->ai_addr;
+					inet_ntop(cur->ai_addr->sa_family,
+					          &sin6->sin6_addr,
+						  buf, sizeof(buf));
+					printf(" %s", buf);
+					if (nscount >= MAXNS)
+						break;
+					u[nscount].sin6 = *sin6;
+					u[nscount++].sin6.sin6_port =
+						htons(NAMESERVER_PORT);
+					break;
+				case AF_INET:
+					sin =
+					     (struct sockaddr_in*)cur->ai_addr;
+					inet_ntop(cur->ai_addr->sa_family,
+					          &sin->sin_addr,
+						  buf, sizeof(buf));
+					printf(" %s", buf);
+					if (nscount >= MAXNS)
+						break;
+					u[nscount].sin = *sin;
+					u[nscount++].sin6.sin6_port =
+						htons(NAMESERVER_PORT);
+					break;
+				}
 			}
-			memcpy(&res.nsaddr.sin_addr, hp->h_addr, NS_INADDRSZ);
-			printf("Using domain server:\n");
-			printanswer(hp);
+			if (nscount != 0) {
+				res_setservers(&res, u, nscount);
+			}
+			if (answer->ai_canonname != NULL)
+				printf("\n\n");
+			else
+				printf(":\n\n");
+			freeaddrinfo(answer);
 		} else {
-			res.nsaddr.sin_family = AF_INET;
-			res.nsaddr.sin_addr = addr;
-			res.nsaddr.sin_port = htons(NAMESERVER_PORT);
-			printf("Using domain server %s:\n",
-			       inet_ntoa(res.nsaddr.sin_addr));
+			fprintf(stderr, "Error in looking up server name:\n");
+			exit(1);
 		}
-		res.nscount = 1;
 		res.retry = 2;
 	}
-	if (strcmp(getdomain, ".") == 0 || !inet_aton(getdomain, &addr))
-		addr.s_addr = INADDR_NONE;
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_flags = AI_NUMERICHOST;
+	hint.ai_socktype = SOCK_DGRAM;
+	if(!getaddrinfo(getdomain, NULL, &hint, &answer)) { 
+		memset(&addr, 0, sizeof(addr));
+		switch (answer->ai_family) {
+		case AF_INET:
+			memcpy(&addr, answer->ai_addr,
+			       sizeof(struct sockaddr_in));
+			ip = 1;
+			break;
+		case AF_INET6:
+			memcpy(&addr, answer->ai_addr,
+			       sizeof(struct sockaddr_in6));
+			ip = 1;
+			break;
+		}
+		freeaddrinfo(answer);
+	} 
 	hp = NULL;
 	res.res_h_errno = TRY_AGAIN;
 /*
@@ -327,7 +394,7 @@ main(int argc, char **argv) {
 		exit(ListHosts(getdomain, querytype ? querytype : ns_t_a));
 	ncnames = 5; nkeychains = 18;
 	while (hp == NULL && res.res_h_errno == TRY_AGAIN) {
-		if (addr.s_addr == INADDR_NONE) {
+		if (!ip) {
 			cname = NULL;
 			hp = (struct hostent *)gethostinfo(getdomain);
 			getdomain[0] = 0; /* clear this query */
@@ -351,7 +418,7 @@ main(int argc, char **argv) {
 					printf ("%s for %s not found, last verified key %s\n",
 						chase_step & SD_SIG ? "Key" : "Signature",
 						chase_step & SD_SIG ? chase_signer : chase_domain, 
-						chase_lastgoodkey ? chase_lastgoodkey : "None");
+						chase_lastgoodkey[0] ? chase_lastgoodkey : "None");
 				}
 			}
 			if (!getdomain[0] && cname) {
@@ -375,7 +442,7 @@ main(int argc, char **argv) {
 				continue;
 			}
 		} else {
-			if (addrinfo(addr) == 0)
+			if (addrinfo(&addr) == 0)
 				hp = NULL;
 			else
 				hp = (struct hostent *)1;	/* XXX */
@@ -424,21 +491,6 @@ parseclass(const char *s) {
 	/*NOTREACHED*/
 }
 
-static void
-printanswer(const struct hostent *hp) {
-	struct in_addr **hptr;
-	char **cp;
-
-	printf("Name: %s\n", hp->h_name);
-	printf("Address:");
-	for (hptr = (struct in_addr **)hp->h_addr_list; *hptr; hptr++)
-		printf(" %s", inet_ntoa(**hptr));
-	printf("\nAliases:");
-	for (cp = hp->h_aliases; cp && *cp && **cp; cp++)
-		printf(" %s", *cp);
-	printf("\n\n");
-}
-
 static void
 hperror(int errnum) {
 	switch(errnum) {
@@ -522,15 +574,50 @@ hperror(int errnum) {
 }
 
 static int
-addrinfo(struct in_addr addr) {
-	u_int32_t ha = ntohl(addr.s_addr);
+addrinfo(struct sockaddr_storage *addr) {
 	char name[NS_MAXDNAME];
+	unsigned char *p;
+	struct in6_addr *addr6;
 
-	sprintf(name, "%u.%u.%u.%u.IN-ADDR.ARPA.",
-		(ha) & 0xff,
-		(ha >> 8) & 0xff,
-		(ha >> 16) & 0xff,
-		(ha >> 24) & 0xff);
+	switch(addr->ss_family) {
+	case AF_INET:
+		p = (unsigned char*)&((struct sockaddr_in *)addr)->sin_addr;
+	mapped:
+		sprintf(name, "%u.%u.%u.%u.IN-ADDR.ARPA.",
+			p[3], p[2], p[1], p[0]);
+		break;
+	case AF_INET6:
+		addr6 = &((struct sockaddr_in6 *)addr)->sin6_addr;
+		p = (unsigned char *)addr6;
+		if (IN6_IS_ADDR_V4MAPPED(addr6) ||
+		    IN6_IS_ADDR_V4COMPAT(addr6)) {
+			p += 12;
+			goto mapped;
+		}
+		sprintf(name,
+			"%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x."
+			"%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x."
+			"IP6.ARPA",
+			p[15] & 0xf, (p[15] >> 4) & 0xf,
+			p[14] & 0xf, (p[14] >> 4) & 0xf,
+			p[13] & 0xf, (p[13] >> 4) & 0xf,
+			p[12] & 0xf, (p[12] >> 4) & 0xf,
+			p[11] & 0xf, (p[11] >> 4) & 0xf,
+			p[10] & 0xf, (p[10] >> 4) & 0xf,
+			p[9] & 0xf, (p[9] >> 4) & 0xf,
+			p[8] & 0xf, (p[8] >> 4) & 0xf,
+			p[7] & 0xf, (p[7] >> 4) & 0xf,
+			p[6] & 0xf, (p[6] >> 4) & 0xf,
+			p[5] & 0xf, (p[5] >> 4) & 0xf,
+			p[4] & 0xf, (p[4] >> 4) & 0xf,
+			p[3] & 0xf, (p[3] >> 4) & 0xf,
+			p[2] & 0xf, (p[2] >> 4) & 0xf,
+			p[1] & 0xf, (p[1] >> 4) & 0xf,
+			p[0] & 0xf, (p[0] >> 4) & 0xf);
+		break;
+	default:
+		abort();
+	}
 	return (getinfo(name, NULL, ns_t_ptr));
 }
 
@@ -588,7 +675,7 @@ gethostinfo(char *name) {
 
 static int
 getdomaininfo(const char *name, const char *domain) {
-	int val1, val2;
+	int val1, val2, val3;
 
 	if (querytype)
 		return (getinfo(name, domain, gettype=querytype));
@@ -596,8 +683,9 @@ getdomaininfo(const char *name, const char *domain) {
 		val1 = getinfo(name, domain, gettype=ns_t_a);
 		if (cname || verbose)
 			return (val1);
-		val2 = getinfo(name, domain, gettype=ns_t_mx);
-		return (val1 || val2);
+		val2 = getinfo(name, domain, gettype=ns_t_aaaa);
+		val3 = getinfo(name, domain, gettype=ns_t_mx);
+		return (val1 || val2 || val3);
 	}
 }
 
@@ -608,7 +696,8 @@ getinfo(const char *name, const char *domain, int type) {
 	int n;
 	char host[NS_MAXDNAME];
 
-	if (domain == NULL)
+	if (domain == NULL ||
+	    (domain[0] == '.' && domain[1] == '\0'))
 		sprintf(host, "%.*s", NS_MAXDNAME, name);
 	else
 		sprintf(host, "%.*s.%.*s",
@@ -630,11 +719,13 @@ getinfo(const char *name, const char *domain, int type) {
 		return (0);
 	}
 	eom = answer.qb2 + n;
-	return (printinfo(&answer, eom, ns_t_any, 0));
+	return (printinfo(&answer, eom, ns_t_any, 0, (type == ns_t_ptr)));
 }
 
 static int
-printinfo(const querybuf *answer, const u_char *eom, int filter, int isls) {
+printinfo(const querybuf *answer, const u_char *eom, int filter, int isls,
+	  int isinaddr)
+{
 	int n, nmx, ancount, nscount, arcount, qdcount, buflen, savesigchase;
 	const u_char *bp, *cp;
 	const HEADER *hp;
@@ -642,7 +733,7 @@ printinfo(const querybuf *answer, const u_char *eom, int filter, int isls) {
 	/*
 	 * Find first satisfactory answer.
 	 */
-	hp = (HEADER *) answer;
+	hp = (const HEADER *) answer;
 	ancount = ntohs(hp->ancount);
 	qdcount = ntohs(hp->qdcount);
 	nscount = ntohs(hp->nscount);
@@ -705,7 +796,7 @@ printinfo(const querybuf *answer, const u_char *eom, int filter, int isls) {
 			 * don't really want to print the address at this
 			 * point.
 			 */
-                        if (cname && ! verbose)
+                        if (cname && (!verbose) && (!isinaddr))
                                 return (1);
                 }
 	}
@@ -720,12 +811,12 @@ printinfo(const querybuf *answer, const u_char *eom, int filter, int isls) {
 	if (nscount) {
 		printf("For authoritative answers, see:\n");
 		while (--nscount >= 0 && cp && cp < eom)
-			cp = (u_char *)pr_rr(cp, answer->qb2, stdout, filter);
+			cp = pr_rr(cp, answer->qb2, stdout, filter);
 	}
 	if (arcount) {
 		printf("Additional information:\n");
 		while (--arcount >= 0 && cp && cp < eom)
-			cp = (u_char *)pr_rr(cp, answer->qb2, stdout, filter);
+			cp = pr_rr(cp, answer->qb2, stdout, filter);
 	}
 
 	/* restore sigchase value */
@@ -735,7 +826,8 @@ printinfo(const querybuf *answer, const u_char *eom, int filter, int isls) {
 	return (1);
 }
 
-void print_hex_field (u_int8_t field[], int length, int width, char *pref)
+static void print_hex_field (u_int8_t field[], int length, int width,
+			     const char *pref)
 {
 	/* Prints an arbitrary bit field, from one address for some number of
 		bytes.  Output is formatted via the width, and includes the raw
@@ -750,12 +842,12 @@ void print_hex_field (u_int8_t field[], int length, int width, char *pref)
 	do
 	{
 		stop=(start+width)<length?(start+width):length;
-		printf (pref);
+		printf ("%s", pref);
  		for (i = start; i < stop; i++)
 			printf ("%02x ", (u_char) field[i]);
 		printf ("\n");
 
-		printf (pref);
+		printf ("%s", pref);
 		for (i = start; i < stop; i++)
 			if (isprint(field[i]))
 				printf (" %c ", (u_char) field[i]);
@@ -767,7 +859,7 @@ void print_hex_field (u_int8_t field[], int length, int width, char *pref)
 	} while (start < length);
 }
 
-void memswap (void *s1, void *s2, size_t n)
+static void memswap (void *s1, void *s2, size_t n)
 {
 	void *tmp;
 
@@ -784,23 +876,6 @@ void memswap (void *s1, void *s2, size_t n)
 	free (tmp);
 }
 
-void print_hex (u_int8_t field[], int length)
-{
-	/* Prints the hex values of a field...not as pretty as the print_hex_field.
-	*/
-	int		i, start, stop;
-
-	start=0;
-	do
-	{
-		stop=length;
- 		for (i = start; i < stop; i++)
-			printf ("%02x ", (u_char) field[i]);
-		start = stop;
-		if (start < length) printf ("\n");
-	} while (start < length);
-}
-
 /*
  * Print resource record fields in human readable form.
  */
@@ -821,7 +896,8 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 	u_char canonrr[MY_PACKETSZ];
 	size_t canonrr_len = 0;
 
-	if ((cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name))) == NULL)
+	cp = pr_cdname(cp, msg, name, sizeof(name));
+	if (cp == NULL)
 		return (NULL);			/* compression error */
 	strcpy(thisdomain, name);
 
@@ -890,7 +966,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		const u_char *startrdata = cp;
 		u_char cdname[NS_MAXCDNAME];
 
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof name);
+		cp = pr_cdname(cp, msg, name, sizeof name);
 		if (doprint)
 			fprintf(file, "%c%s", punc, name);
 
@@ -939,7 +1015,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		const u_char *startname = cp;
 		u_char cdname[NS_MAXCDNAME];
 
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof name);
+		cp = pr_cdname(cp, msg, name, sizeof name);
 		if (doprint)
 			fprintf(file, "\t%s", name);
 
@@ -956,7 +1032,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		}
 
 		startname = cp;
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof name);
+		cp = pr_cdname(cp, msg, name, sizeof name);
 		if (doprint)
 			fprintf(file, " %s", name);
 
@@ -1015,7 +1091,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 				fprintf(file," ");
 		}
 		cp += sizeof(u_short);
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
+		cp = pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint)
 			fprintf(file, "%s", name);
 
@@ -1051,7 +1127,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		if (doprint)
 			fprintf(file," %d", ns_get16(cp));
 		cp += sizeof(u_short);
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
+		cp = pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint)
 			fprintf(file, " %s", name);
 		break;
@@ -1093,14 +1169,14 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		}
 		cp += n;
 		/* replacement  */
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
+		cp = pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint)
 			fprintf(file, "%s", name);
 		break;
 
 	case ns_t_minfo:
 	case ns_t_rp:
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof name);
+		cp = pr_cdname(cp, msg, name, sizeof name);
 		if (doprint) {
 			if (type == ns_t_rp) {
 				char *p;
@@ -1111,7 +1187,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 			}
 			fprintf(file, "%c%s", punc, name);
 		}
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
+		cp = pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint)
 			fprintf(file, " %s", name);
 		break;
@@ -1196,7 +1272,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 		u_char cdname[NS_MAXCDNAME];
 		size_t bitmaplen;
 
-		cp = (u_char *) pr_cdname(cp, msg, name, sizeof name);
+		cp = pr_cdname(cp, msg, name, sizeof name);
 		if (doprint)
 			fprintf(file, "%c%s", punc, name);
 		bitmaplen = dlen - (cp - startrdata);
@@ -1260,7 +1336,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 			fprintf(file, " %d", ns_get16(cp));
 		cp += sizeof(u_short);
 		/* signer's name */
-		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
+		cp = pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint && verbose)
 			fprintf(file, " %s", name);
 		else if (doprint && !verbose)
@@ -1294,7 +1370,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 					       SIG_RDATA_BY_NAME);
 					memcpy(chase_sigrdata + SIG_RDATA_BY_NAME,
 					       cdname, n);
-					chase_sigrdata_len += SIG_RDATA_BY_NAME + n;
+					chase_sigrdata_len = SIG_RDATA_BY_NAME + n;
 					memcpy(chase_signature, cp, len);
 					chase_signature_len = len;
 
@@ -1371,10 +1447,14 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 			/* sort rr's (qsort() is too slow) */
 			
 			for (i = 0; i < NUMRR && chase_rr[i].len; i++)
-				for (j = i + 1; i < NUMRR && chase_rr[j].len; j++)
-					if (memcmp(chase_rr[i].data, chase_rr[j].data, MY_PACKETSZ) > 0)
-						memswap(&chase_rr[i], &chase_rr[j], sizeof(rrstruct));
-			
+				for (j = i + 1; j < NUMRR && chase_rr[j].len;
+				     j++)
+					if (memcmp(chase_rr[i].data,
+						   chase_rr[j].data,
+						   MY_PACKETSZ) < 0)
+						memswap(&chase_rr[i],
+						        &chase_rr[j],
+							sizeof(rrstruct));
 			/* append rr's to sigrdata */
 
 			for (i = 0; i < NUMRR && chase_rr[i].len; i++)
@@ -1387,6 +1467,7 @@ pr_rr(const u_char *cp, const u_char *msg, FILE *file, int filter) {
 			/* print rr-data and signature */
 
 			if (verbose) {
+				fprintf(file, "\n");
 				print_hex_field(chase_sigrdata,
 						chase_sigrdata_len,
 						21,"DATA: ");
@@ -1502,6 +1583,7 @@ static const char *
 pr_type(int type) {
 	if (!verbose) switch (type) {
 	case ns_t_a:
+	case ns_t_aaaa:
 		return ("has address");
 	case ns_t_cname:
 		return ("is a nickname for");
@@ -1556,12 +1638,44 @@ pr_cdname(const u_char *cp, const u_char *msg, char *name, int namelen) {
 	return (cp + n);
 }
 
+static void
+add(union res_sockaddr_union *u, int type, void *p) {
+	memset(u, 0, sizeof(*u));
+	switch (type) {
+	case ns_t_a:
+		memcpy(&u->sin.sin_addr, p, NS_INADDRSZ);
+		u->sin.sin_family = AF_INET;
+		u->sin.sin_port = htons(NAMESERVER_PORT);
+#ifdef HAVE_SA_LEN
+		u->sin.sin_len = sizeof(u->sin);
+#endif
+		break;
+
+	case ns_t_aaaa:
+		memcpy(&u->sin6.sin6_addr, p, 16);
+		u->sin6.sin6_family = AF_INET6;
+		u->sin6.sin6_port = htons(NAMESERVER_PORT);
+#ifdef HAVE_SA_LEN
+		u->sin6.sin6_len = sizeof(u->sin6);
+#endif
+		break;
+	}
+}
+
+static int
+salen(union res_sockaddr_union *u) {
+	switch (u->sin.sin_family) {
+	case AF_INET6: return (sizeof(u->sin6));
+	case AF_INET: return (sizeof(u->sin));
+	}
+	return (0);
+}
+
 static int
 ListHosts(char *namePtr, int queryType) {
 	querybuf buf, answer;
 	struct sockaddr_in sin;
 	const HEADER *headerPtr;
-	const struct hostent *hp;
 	enum { NO_ERRORS, ERR_READING_LEN, ERR_READING_MSG, ERR_PRINTING }
 		error = NO_ERRORS;
 
@@ -1574,7 +1688,7 @@ ListHosts(char *namePtr, int queryType) {
 	/* Names and addresses of name servers to try. */
 	char nsname[NUMNS][NS_MAXDNAME];
 	int nshaveaddr[NUMNS];
-	struct in_addr nsipaddr[NUMNSADDR];
+	union res_sockaddr_union nsipaddr[NUMNSADDR];
 	int numns, numnsaddr, thisns;
 	int qdcount, ancount;
 
@@ -1586,10 +1700,9 @@ ListHosts(char *namePtr, int queryType) {
 	if (namePtr[i-1] == '.')
 		namePtr[i-1] = 0;
 
-	if (server_specified) {
-		memcpy(&nsipaddr[0], &res.nsaddr.sin_addr, NS_INADDRSZ);
-		numnsaddr = 1;
-	} else {
+	if (server_specified)
+		numnsaddr = res_getservers(&res, nsipaddr, NUMNSADDR);
+	else {
 		/*
 		 * First we have to find out where to look.  This needs a NS
 		 * query, possibly followed by looking up addresses for some
@@ -1707,20 +1820,17 @@ ListHosts(char *namePtr, int queryType) {
 						}
 					}
 				}
-			} else if (type == ns_t_a) {
-				if (numnsaddr < NUMNSADDR)
-					for (i = 0; i < numns; i++) {
-						if (ns_samename(nsname[i],
+			} else if ((type == ns_t_a || type == ns_t_aaaa) &&
+				   numnsaddr < NUMNSADDR) {
+				for (i = 0; i < numns; i++) {
+					if (ns_samename(nsname[i],
 							       (char *)domain)
-						    == 1) {
-							nshaveaddr[i]++;
-							memcpy(
-							  &nsipaddr[numnsaddr],
-							  cp, NS_INADDRSZ);
-							numnsaddr++;
-							break;
-						}
-					}
+					    != 1)
+						continue;
+					nshaveaddr[i]++;
+					add(&nsipaddr[numnsaddr++], type, cp);
+					break;
+				}
 			}
 			cp += dlen;
 		}
@@ -1732,28 +1842,50 @@ ListHosts(char *namePtr, int queryType) {
 		 */
 
 		for (i = 0; i < numns; i++) {
-			if (nshaveaddr[i] == 0) {
-				struct in_addr **hptr;
-				int numaddrs = 0;
+			struct addrinfo *answer = NULL;
+			struct addrinfo *cur = NULL;
+			struct addrinfo hint;
 
-				hp = gethostbyname(nsname[i]);
-				if (hp) {
-					for (hptr = (struct in_addr **)
-					     	hp->h_addr_list;
-					     *hptr != NULL;
-					     hptr++)
-						if (numnsaddr < NUMNSADDR) {
-							memcpy(
-							  &nsipaddr[numnsaddr],
-							  *hptr, NS_INADDRSZ);
-							numnsaddr++;
-							numaddrs++;
-						}
+			memset(&hint, 0, sizeof(hint));
+			hint.ai_family = PF_UNSPEC;
+			hint.ai_socktype = SOCK_STREAM;
+
+			if (nshaveaddr[i] == 0 &&
+			    !getaddrinfo(nsname[i], NULL, &hint, &answer)) {
+				int numaddrs = 0;
+				for (cur = answer;
+				     cur != NULL;
+				     cur = cur->ai_next) {
+					union res_sockaddr_union *u;
+
+					if (numnsaddr >= NUMNSADDR)
+						break;
+
+					u = &nsipaddr[numnsaddr];
+					switch (cur->ai_addr->sa_family) {
+					case AF_INET6:
+						u->sin6 =
+					   *(struct sockaddr_in6 *)cur->ai_addr;
+						u->sin6.sin6_port =
+							htons(NAMESERVER_PORT);
+						numnsaddr++;
+						numaddrs++;
+						break;
+					case AF_INET:
+						u->sin =
+					    *(struct sockaddr_in*)cur->ai_addr;
+						u->sin6.sin6_port =
+							htons(NAMESERVER_PORT);
+						numnsaddr++;
+						numaddrs++;
+						break;
+					}
 				}
 				if (res.options & RES_DEBUG || verbose)
 					printf(
 				  "Found %d addresses for %s by extra query\n",
 					       numaddrs, nsname[i]);
+				freeaddrinfo(answer);
 			} else if (res.options & RES_DEBUG || verbose)
 				printf("Found %d addresses for %s\n",
 				       nshaveaddr[i], nsname[i]);
@@ -1790,14 +1922,31 @@ ListHosts(char *namePtr, int queryType) {
 	 */
 
 	for ((void)NULL; thisns < numnsaddr; thisns++) {
-		if ((sockFD = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+		if ((sockFD = socket(nsipaddr[thisns].sin.sin_family,
+				     SOCK_STREAM, 0)) < 0) {
+			if (errno == EPROTONOSUPPORT)
+				continue;
 			perror("ListHosts");
 			return (ERROR);
 		}
-		memcpy(&sin.sin_addr, &nsipaddr[thisns], NS_INADDRSZ);
-		if (res.options & RES_DEBUG || verbose)
-			printf("Trying %s\n", inet_ntoa(sin.sin_addr));
-		if (connect(sockFD, (struct sockaddr *)&sin, sizeof(sin)) >= 0)
+		if (res.options & RES_DEBUG || verbose) {
+			char buf[80];
+			switch (nsipaddr[thisns].sin.sin_family) {
+			case AF_INET:
+				inet_ntop(nsipaddr[thisns].sin.sin_family,
+					  &nsipaddr[thisns].sin.sin_addr,
+					  buf, sizeof(buf));
+				break;
+			case AF_INET6:
+				inet_ntop(nsipaddr[thisns].sin6.sin6_family,
+					  &nsipaddr[thisns].sin6.sin6_addr,
+					  buf, sizeof(buf));
+				break;
+			}
+			printf("Trying %s\n", buf);
+		}
+		if (connect(sockFD, (struct sockaddr *)&nsipaddr[thisns],
+		            salen(&nsipaddr[thisns])) >= 0)
 			break;
 		if (verbose)
 			perror("Connection failed, trying next server");
@@ -1881,7 +2030,7 @@ ListHosts(char *namePtr, int queryType) {
 			break;
 		}
 
-		result = printinfo(&buf, cp, queryType, 1);
+		result = printinfo(&buf, cp, queryType, 1, 0);
 		if (! result) {
 			error = ERR_PRINTING;
 			break;
