@@ -1,5 +1,5 @@
 #ifndef LINT
-static const char rcsid[] = "$Header: /proj/cvs/isc/bind8/src/lib/dst/dst_api.c,v 1.16 2000/11/13 04:09:23 vixie Exp $";
+static const char rcsid[] = "$Header: /proj/cvs/isc/bind8/src/lib/dst/dst_api.c,v 1.20 2001/07/26 01:20:08 marka Exp $";
 #endif
 
 /*
@@ -62,8 +62,8 @@ static const char rcsid[] = "$Header: /proj/cvs/isc/bind8/src/lib/dst/dst_api.c,
 /* static variables */
 static int done_init = 0;
 dst_func *dst_t_func[DST_MAX_ALGS];
-char *key_file_fmt_str = "Private-key-format: v%s\nAlgorithm: %d (%s)\n";
-char *dst_path = "";
+const char *key_file_fmt_str = "Private-key-format: v%s\nAlgorithm: %d (%s)\n";
+const char *dst_path = "";
 
 /* internal I/O functions */
 static DST_KEY *dst_s_read_public_key(const char *in_name, 
@@ -109,12 +109,14 @@ dst_init()
 		} else if (stat(s, &statbuf) != 0 || !S_ISDIR(statbuf.st_mode)) {
 			EREPORT(("%s is not a valid directory\n", s));
 		} else {
-			dst_path = (char *) malloc(len + 2);
-			memcpy(dst_path, s, len + 1);
-			if (dst_path[strlen(dst_path) - 1] != '/') {
-				dst_path[strlen(dst_path) + 1] = 0;
-				dst_path[strlen(dst_path)] = '/';
+			char *tmp;
+			tmp = (char *) malloc(len + 2);
+			memcpy(tmp, s, len + 1);
+			if (tmp[strlen(tmp) - 1] != '/') {
+				tmp[strlen(tmp) + 1] = 0;
+				tmp[strlen(tmp)] = '/';
 			}
+			dst_path = tmp;
 		}
 	}
 	memset(dst_t_func, 0, sizeof(dst_t_func));
@@ -433,7 +435,7 @@ dst_s_write_private_key(const DST_KEY *key)
 		int nn;
 		if ((nn = fwrite(encoded_block, 1, len, fp)) != len) {
 			EREPORT(("dst_write_private_key(): Write failure on %s %d != %d errno=%d\n",
-				 file, out_len, nn, errno));
+				 file, len, nn, errno));
 			return (-5);
 		}
 		fclose(fp);
@@ -555,7 +557,7 @@ dst_s_read_public_key(const char *in_name, const u_int16_t in_id, int in_alg)
 	enckey[--len] = '\0';
 
 	/* remove leading spaces */
-	for (notspace = (char *) enckey; isspace(*notspace); len--)
+	for (notspace = (char *) enckey; isspace((*notspace)&0xff); len--)
 		notspace++;
 
 	dlen = b64_pton(notspace, deckey, sizeof(deckey));
@@ -589,6 +591,7 @@ dst_s_write_public_key(const DST_KEY *key)
 	u_char out_key[RAW_KEY_SIZE];
 	char enc_key[RAW_KEY_SIZE];
 	int len = 0;
+	int mode;
 
 	memset(out_key, 0, sizeof(out_key));
 	if (key == NULL) {
@@ -604,8 +607,10 @@ dst_s_write_public_key(const DST_KEY *key)
 			 key->dk_key_name, key->dk_id, PUBLIC_KEY));
 		return (0);
 	}
+	/* XXX in general this should be a check for symmetric keys */
+	mode = (key->dk_alg == KEY_HMAC_MD5) ? 0600 : 0644;
 	/* create public key file */
-	if ((fp = dst_s_fopen(filename, "w+", 0644)) == NULL) {
+	if ((fp = dst_s_fopen(filename, "w+", mode)) == NULL) {
 		EREPORT(("DST_write_public_key: open of file:%s failed (errno=%d)\n",
 			 filename, errno));
 		return (0);
@@ -657,6 +662,7 @@ dst_dnskey_to_key(const char *in_name, const u_char *rdata, const int len)
 
 	if (in_name == NULL)
 		return (NULL);
+	key_st->dk_id = dst_s_dns_key_id(rdata, len);
 	key_st->dk_flags = dst_s_get_int16(rdata);
 	key_st->dk_proto = (u_int16_t) rdata[DST_KEY_PROT];
 	if (key_st->dk_flags & DST_EXTEND_FLAG) {
@@ -760,6 +766,8 @@ dst_buffer_to_key(const char *key_name,		/* name of the key */
 {
 	
 	DST_KEY *dkey = NULL; 
+	int dnslen;
+	u_char dns[2048];
 
 	if (!dst_check_algorithm(alg)) { /* make sure alg is available */
 		EREPORT(("dst_buffer_to_key(): Algorithm %d not suppored\n", alg));
@@ -771,14 +779,17 @@ dst_buffer_to_key(const char *key_name,		/* name of the key */
 
 	if (dkey == NULL)
 		return (NULL);
-	if (dkey->dk_func != NULL && dkey->dk_func->from_dns_key != NULL) {
-		if (dkey->dk_func->from_dns_key(dkey, key_buf, key_len) < 0) {
-			EREPORT(("dst_buffer_to_key(): dst_buffer_to_hmac failed\n"));
-			return (dst_free_key(dkey));
-		}
-		return (dkey);
+	if (dkey->dk_func == NULL || dkey->dk_func->from_dns_key == NULL)
+		return NULL;
+
+	if (dkey->dk_func->from_dns_key(dkey, key_buf, key_len) < 0) {
+		EREPORT(("dst_buffer_to_key(): dst_buffer_to_hmac failed\n"));
+		return (dst_free_key(dkey));
 	}
-	return (NULL);
+
+	dnslen = dst_key_to_dnskey(dkey, dns, sizeof(dns));
+	dkey->dk_id = dst_s_dns_key_id(dns, dnslen);
+	return (dkey);
 }
 
 int 
@@ -816,10 +827,12 @@ dst_s_read_private_key_file(char *name, DST_KEY *pk_key, u_int16_t in_id,
 			    int in_alg)
 {
 	int cnt, alg, len, major, minor, file_major, file_minor;
-	int id;
+	int ret, id;
 	char filename[PATH_MAX];
 	u_char in_buff[RAW_KEY_SIZE], *p;
 	FILE *fp;
+	int dnslen;
+	u_char dns[2048];
 
 	if (name == NULL || pk_key == NULL) {
 		EREPORT(("dst_read_private_key_file(): No key name given\n"));
@@ -886,10 +899,13 @@ dst_s_read_private_key_file(char *name, DST_KEY *pk_key, u_int16_t in_id,
 	if (pk_key->dk_func == NULL || pk_key->dk_func->from_file_fmt == NULL)
 		goto fail;
 
-	id = pk_key->dk_func->from_file_fmt(pk_key, (char *)p, &in_buff[len] - p);
-	if (id < 0)
+	ret = pk_key->dk_func->from_file_fmt(pk_key, (char *)p, &in_buff[len] - p);
+	if (ret < 0)
 		goto fail;
 
+	dnslen = dst_key_to_dnskey(pk_key, dns, sizeof(dns));
+	id = dst_s_dns_key_id(dns, dnslen);
+
 	/* Make sure the actual key tag matches the input tag used in the filename
 	 */
 	if (id != in_id) {
@@ -942,6 +958,9 @@ dst_generate_key(const char *name, const int bits, const int exp,
 {
 	DST_KEY *new_key = NULL;
 	int res;
+	int dnslen;
+	u_char dns[2048];
+
 	if (name == NULL)
 		return (NULL);
 
@@ -966,6 +985,13 @@ dst_generate_key(const char *name, const int bits, const int exp,
 			 new_key->dk_key_size, exp));
 		return (dst_free_key(new_key));
 	}
+
+	dnslen = dst_key_to_dnskey(new_key, dns, sizeof(dns));
+	if (dnslen != UNSUPPORTED_KEYALG)
+		new_key->dk_id = dst_s_dns_key_id(dns, dnslen);
+	else
+		new_key->dk_id = 0;
+
 	return (new_key);
 }
 
