@@ -1,7 +1,7 @@
 #ifndef _PRAND_CMD_H_
 #define _PRAND_CMD_H_
 
-const char *cmds[] = {
+static const char *cmds[] = {
 	"/bin/ps -ef 2>&1",
 	"/etc/arp -n -a 2>&1",
 	"/usr/bin/netstat -an 2>&1",
@@ -11,7 +11,7 @@ const char *cmds[] = {
 	NULL
 };
 
-const char *dirs[] = {
+static const char *dirs[] = {
 	"/tmp",
 	"/usr/tmp",
 	".",
@@ -22,7 +22,7 @@ const char *dirs[] = {
 	NULL
 };
 
-const char *files[] = {
+static const char *files[] = {
 	NULL
 };
 
